package cn.iocoder.yudao.module.bpm.api;

import cn.iocoder.yudao.module.bpm.resp.TaskResp;
import org.springframework.stereotype.Service;

/**
 * 流程创建工厂
 */
@Service
public interface BpmCreateServiceFactory {


    /**
     * 用法是表单对象叫一定要创建一个字段 process_instance_id，用来存储流程创建后返回的ID，字段名称不能变
     *
     * @param userId      用户ID
     * @param businessId  业务主键-流程回调时会返回，根据此ID反查业务数据进行处理
     * @param workFlowKey WorkFlowEmus枚举下的KEY--创建哪个流程
     * @param businessNo  业务单据号
     * @param copyUserId  抄送人
     * @return 流程实例ID，需回写到业务对象中--用于发起流程取消，和查询当前流程审批节点
     */
    String createBmp(Long userId, Long businessId, String workFlowKey, String businessNo, String[] copyUserId);


    /**
     * 用法是表单对象叫一定要创建一个字段 process_instance_id，用来存储流程创建后返回的ID，字段名称不能变
     *
     * @param userId      用户ID
     * @param businessId  业务主键-流程回调时会返回，根据此ID反查业务数据进行处理
     * @param workFlowKey WorkFlowEmus枚举下的KEY--创建哪个流程
     * @param businessNo  业务单据号
     * @param copyUserId  抄送人
     * @param designatedReview  指定审核人
     * @return 流程实例ID，需回写到业务对象中--用于发起流程取消，和查询当前流程审批节点
     */
    String createBmpDesignated(Long userId, Long businessId, String workFlowKey, String businessNo, String[] copyUserId, String designatedReview);

    /**
     * 取消流程
     *
     * @param userId               取消人
     * @param bpmProcessInstanceId 流程实例ID
     * @param reason               取消原因
     */
    void cancelBmpById(Long userId, String bpmProcessInstanceId, String reason);

    /**
     * 根据流程ID返回此流程的最后一条审批记录
     *
     * @param bpmProcessInstanceId 流程实例ID
     * @return
     */
    TaskResp getLastTaskByProcessInstanceId(String bpmProcessInstanceId);
}
